package aCupOfCoffee;

import java.awt.*;

class TransparentPanel extends Panel {

	public void update (Graphics g)
	{
		paint (g);
	}
	public void paint (Graphics g)
	{
	}
}

class BoxBorder extends Canvas {

	private Box box;
	int realX, realY;
	
	public BoxBorder (Box aBox)
	{
		box = aBox;
	}
	
	public void move (int x, int y)
	{
		realX = x;
		realY = y;
		super.move (x, y);
	}
	
	public void paint (Graphics g)
	{	
		Rectangle bounds;
		Graphics pg;
		
		bounds = bounds ();
		pg = box.getGraphics ();
		pg.clipRect (realX, realY, bounds.width, bounds.height);
		box.drawBox (g, -realX, -realY);
	}
}

class BoxLayout extends Object implements LayoutManager {

	private TransparentPanel content;
	private BoxBorder leftBorder;
	private BoxBorder topBorder;
	private BoxBorder rightBorder;
	private BoxBorder bottomBorder;

	public BoxLayout (TransparentPanel cont,
					  BoxBorder left, BoxBorder top,
					  BoxBorder right, BoxBorder bottom)
	{
		content = cont;
		leftBorder = left;
		topBorder = top;
		rightBorder = right;
		bottomBorder = bottom;
	}

	public void layoutContainer (Container target)
	{
		Insets insets;
		Dimension targetSize;
		Dimension minContentSize;
		int contentWidth;
		int contentHeight;
		boolean needsEnlargement;
		
		targetSize = target.size ();
		((Box)target).calculateLayout ();
		insets = target.insets();
		contentWidth = targetSize.width - insets.left - insets.right;
		contentHeight = targetSize.height - insets.top - insets.bottom;		
		content.resize      (contentWidth, contentHeight);
		content.move        (insets.left, insets.top);
		leftBorder.resize   (insets.left, contentHeight);
		leftBorder.move     (0, insets.top);
		rightBorder.resize  (insets.right, contentHeight);
		rightBorder.move    (insets.left + contentWidth, insets.top);
		topBorder.resize    (targetSize.width, insets.top);
		topBorder.move      (0, 0);
		bottomBorder.resize (targetSize.width, insets.bottom);
		bottomBorder.move   (0, insets.top + contentHeight); 
	}

	public void addLayoutComponent (String name, Component comp)
	{
	}
	
	public void removeLayoutComponent (Component comp)
	{
	}

	public Dimension preferredLayoutSize(Container target)
	{
		Dimension cSize;
		Insets insets;
		
		cSize = content.preferredSize ();	
		insets = target.insets ();
		return new Dimension (cSize.width + insets.left + insets.right,
							  cSize.height + insets.top + insets.bottom);
	}

	public Dimension minimumLayoutSize(Container target)
	{
		Dimension cSize;
		Insets insets;
		
		cSize = content.minimumSize ();	
		insets = target.insets ();
		return new Dimension (cSize.width + insets.left + insets.right,
							  cSize.height + insets.top + insets.bottom);
	}
}

public class Box extends TransparentPanel {

	public final static String copyright =
	"aCupOfCoffee (c) 1996 melonSoft Ralf Suckow Berlin. All Rights Reserved";  

	public final static int NOBORDER = 0;
	public final static int LINE     = 1;
	public final static int BEZEL    = 2;
	public final static int GROOVE   = 3;
	
	public final static int NOTITLE     = 0;
	public final static int ABOVETOP    = 1;
	public final static int ATTOP       = 2;
	public final static int BELOWTOP    = 3;
	public final static int ABOVEBOTTOM = 4;
	public final static int ATBOTTOM    = 5;
	public final static int BELOWBOTTOM = 6;
	
	private final static int HORIZTITLEOFFSET = 4;
	
	// subcomponents
	private TransparentPanel content;
	private BoxBorder leftBorder;
	private BoxBorder topBorder;
	private BoxBorder rightBorder;
	private BoxBorder bottomBorder;
	
	// permanent attributes
	private String		title;
	private int			titlePosition;
	private int			borderType;
	private int			horizOffset;
	private int			vertOffset;
	
	// temporary attributes derived in layout
	private Insets		insets;
	private Rectangle	titleRect;
	private Rectangle	borderRect;
	private Point		titleOrigin;
	
	public Box (String theTitle, int theTitlePosition, int theBorderType,
				int hOffset, int vOffset)
	{
		content = new TransparentPanel ();
		leftBorder = new BoxBorder (this);
		topBorder = new BoxBorder (this);
		rightBorder = new BoxBorder (this);
		bottomBorder = new BoxBorder (this);

        super.setLayout (new BoxLayout (content, leftBorder, topBorder,
										rightBorder, bottomBorder));
		
		super.add (content);
		super.add (leftBorder);
		super.add (topBorder);
		super.add (rightBorder);
		super.add (bottomBorder);

		title = theTitle;
		titlePosition = theTitlePosition;
		borderType = theBorderType;
		horizOffset = hOffset;
		vertOffset = vOffset;
		
		insets = new Insets (0, 0, 0, 0);
		titleRect = new Rectangle ();
		borderRect = new Rectangle ();
		titleOrigin = new Point (0, 0);
		invalidate ();

	}

	public Component add (Component comp)
	{
		if (content == null)
			return super.add (comp);
		else
			return content.add (comp);
	}
	
	public void setLayout(LayoutManager mgr)
	{
		if (content == null)
			super.setLayout (mgr);
		else
			content.setLayout (mgr);
	}

	public Box ()
	{
		this ("Box", ATTOP, GROOVE, 5, 5);
	}

	public void calculateLayout ()
	{
		// we are calculating:
		// insets
		// titleRect
		// titleOrigin
		// borderRect
		
		// the input is the own bounds and the font size 
		// as well as the title
		
		Rectangle	bounds;
		int			borderSize;
		FontMetrics fontMetrics;
		Font		font;
		double      fontSize;
		int			ascent;
		int			a, b, X;
		
		bounds = bounds ();
		font = getFont ();

		if (font == null)
			// not yet on the screen
			return;

		fontMetrics = getFontMetrics (font);
		if (fontMetrics == null)
		  return; // not yet on screen
		
		ascent = fontMetrics.getMaxAscent ();
		// 1.0 final:
		// fontSize = (double) (fontMetrics.getMaxDescent () + ascent);
		// 1.0 prebeta 1:
		fontSize = (double) (fontMetrics.getMaxDecent () + ascent);

		switch (borderType) {
		case NOBORDER:	borderSize = 0; break;
		case LINE:		borderSize = 1; break;
		case BEZEL:		borderSize = 2; break;
		case GROOVE:	borderSize = 2; break;
		default:		borderSize = 0;
		}
		
		// horizontal settings
		insets.left = borderSize + horizOffset;
		insets.right = borderSize + horizOffset;
		titleRect.width = fontMetrics.stringWidth (title)
							+ 2 * HORIZTITLEOFFSET;
		titleRect.x = (bounds.width - titleRect.width) / 2;
		titleOrigin.x = titleRect.x + HORIZTITLEOFFSET;
		borderRect.x = 0;
		borderRect.width = bounds.width;
		
		// vertical settings
		switch (titlePosition) {
		case NOTITLE:
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = 0;
			titleRect.y = 0;
			titleOrigin.y = 0;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ABOVETOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			
			insets.top = b + borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0;
			titleOrigin.y = a;
			borderRect.y = b;
			borderRect.height = bounds.height - b;
			break;
			
		case ATTOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			X = (int) Math.round (0.5834 * fontSize);
			
			insets.top = b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0;
			titleOrigin.y = a;
			borderRect.y = X;
			borderRect.height = bounds.height - X;
			break;
			
		case BELOWTOP:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset + b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = borderSize;
			titleOrigin.y = borderSize + a;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ABOVEBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + b;
			titleRect.height = b;
			titleRect.y = bounds.height - insets.bottom;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ATBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			b = (int) Math.round (0.5834 * fontSize);
			X = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = X;
			titleRect.height = X;
			titleRect.y = bounds.height - X;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height - b;
			break;
			
		case BELOWBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset + b;
			titleRect.height = b;
			titleRect.y = bounds.height - b;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height - b; // F
			break;

		default: break; // ??
		}
	}
	
	public String title ()
	{
		return title;
	}
	
	public void setTitle (String newTitle)
	{
		invalidate ();
		title = newTitle;
	}
	
	public int titlePosition ()
	{
		return titlePosition;
	}
	
	public void setTitlePosition (int newPosition)
	{
		invalidate ();
		titlePosition = newPosition;
	}
	
	public int borderType ()
	{
		return borderType;
	}
	
	public void setBorderType (int newBorderType)
	{
		invalidate ();
		borderType = newBorderType;
	}
	
	public int horizOffset ()
	{
		return horizOffset;
	}
	
	public void setHorizOffset (int offset)
	{
		invalidate ();
		horizOffset = offset;
	}
	
	public int vertOffset ()
	{
		return vertOffset;
	}
	
	public void setVertOffset (int offset)
	{
		invalidate ();
		vertOffset = offset;
	}
	
	public Insets insets()
	{
		return insets;
	}
	
	private void drawLineRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.black);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + r.height - 1, 
					r.x, r.y + r.height - 1);
		g.drawLine (r.x, r.y + r.height - 1, 
					r.x, r.y);
	}
	
	private void drawBezelRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 2, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 2);
		g.setColor (Color.black);
		g.drawLine (r.x + 2, r.y + 1,
					r.x + r.width - 3, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 3);
		g.setColor (Color.lightGray);
		g.drawLine (r.x + 1, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 1,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawGrooveRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + r.width - 1, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + 2,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawText (Graphics g, Point p, String text)
	{
		g.setFont (getFont ());
		g.setColor (getForeground ());
		g.drawString (text, p.x, p.y);
	}
	
	public void drawBox (Graphics g, int xOffset, int yOffset)
	{


		// g.clearRect (0, 0, bounds.width, bounds.height);

		Rectangle offsetBorderRect;

		offsetBorderRect = new Rectangle (borderRect.x + xOffset,
										  borderRect.y + yOffset,
										  borderRect.width,
										  borderRect.height);
		
		switch (borderType) {
		case NOBORDER: break;
		case LINE:     drawLineRect (g, offsetBorderRect); break;
		case BEZEL:    drawBezelRect (g, offsetBorderRect); break;
		case GROOVE:   drawGrooveRect (g, offsetBorderRect); break;
		}
		
		if (titlePosition == NOTITLE)
		  return;
		  
		g.clearRect (titleRect.x + xOffset, titleRect.y + yOffset, 
					 titleRect.width, titleRect.height);

		if (title != null && title.length () > 0)
			drawText (g, new Point (titleOrigin.x + xOffset,
									titleOrigin.y + yOffset), title);
	}
}
