/*
 * Copyright (c) 1996 melonSoft Ralf Suckow Berlin. All Rights Reserved.
 *
 * MELONSOFT RALF SUCKOW BERLIN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. MELONSOFT RALF SUCKOW BERLIN 
 * SHALL NOT BE LIABLE FOR ANY DAMAGES RESULTING FROM USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package aCupOfCoffee;

import java.awt.*;

public class Box extends Panel {

	public final static String copyright =
			"(c) 1996 melonSoft Ralf Suckow Berlin. All Rights Reserved";  

	public final static int NOBORDER = 0;
	public final static int LINE     = 1;
	public final static int BEZEL    = 2;
	public final static int GROOVE   = 3;
	
	public final static int NOTITLE     = 0;
	public final static int ABOVETOP    = 1;
	public final static int ATTOP       = 2;
	public final static int BELOWTOP    = 3;
	public final static int ABOVEBOTTOM = 4;
	public final static int ATBOTTOM    = 5;
	public final static int BELOWBOTTOM = 6;
	
	private final static int HORIZTITLEOFFSET = 4;
	
	// permanent attributes
	private String		title;
	private int			titlePosition;
	private int			borderType;
	private int			horizOffset;
	private int			vertOffset;
	
	// temporary attributes derived in calculateGeometry
	private Insets		insets;
	private Rectangle	titleRect;
	private Rectangle	borderRect;
	private Point		titleOrigin;
	
	public Box (String theTitle, int theTitlePosition, int theBorderType,
				int hOffset, int vOffset)
	{
		title = theTitle;
		titlePosition = theTitlePosition;
		borderType = theBorderType;
		horizOffset = hOffset;
		vertOffset = vOffset;
		
		insets = new Insets (0, 0, 0, 0);
		titleRect = new Rectangle ();
		borderRect = new Rectangle ();
		titleOrigin = new Point (0, 0);

		resize (minimumSize ());
		setBackground (Color.lightGray);
		setForeground (Color.black);
	}

	public Box ()
	{
		this ("Box", ATTOP, GROOVE, 5, 5);
	}

	private void calculateGeometry ()
	{
		// we are calculating:
		// insets
		// titleRect
		// titleOrigin
		// borderRect
		
		// the input is the own bounds and the font size 
		// as well as the title
		
		Rectangle	bounds;
		int			borderSize;
		FontMetrics fontMetrics;
		Font		font;
		double      fontSize;
		int			ascent;
		int			a, b, X; // temporary values
		
		bounds = bounds ();
		font = getFont ();

		if (font == null)
			// not yet on the screen
			return;

		fontMetrics = getFontMetrics (font);
		if (fontMetrics == null)
		  return; // not yet on screen
		ascent = fontMetrics.getMaxAscent ();
		// 1.0 final fontSize = (double) (fontMetrics.getMaxDescent () + ascent);
		// 1.0 prebeta 1
		fontSize = (double) (fontMetrics.getMaxDecent () + ascent);

		switch (borderType) {
		case NOBORDER:	borderSize = 0; break;
		case LINE:		borderSize = 1; break;
		case BEZEL:		borderSize = 2; break;
		case GROOVE:	borderSize = 2; break;
		default:		borderSize = 0;
		}
		
		// horizontal settings
		insets.left = borderSize + horizOffset;
		insets.right = borderSize + horizOffset;
		titleRect.width = fontMetrics.stringWidth (title)
							+ 2 * HORIZTITLEOFFSET; // CHECK
		titleRect.x = (bounds.width - titleRect.width) / 2; // + bounds.x;
		titleOrigin.x = titleRect.x + HORIZTITLEOFFSET;
		borderRect.x = 0; // bounds.x;
		borderRect.width = bounds.width;
		
		// vertical settings
		switch (titlePosition) {
		case NOTITLE:
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = 0;
			titleRect.y = 0;
			titleOrigin.y = 0;
			borderRect.y = 0; // bounds.y;
			borderRect.height = bounds.height;
			break;
			
		case ABOVETOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			
			insets.top = b + borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0; // bounds.y;
			titleOrigin.y = a; // bounds.y + a;
			borderRect.y = b; // bounds.y + b;
			borderRect.height = bounds.height - b;
			break;
			
		case ATTOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			X = (int) Math.round (0.5834 * fontSize);
			
			insets.top = b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0; // bounds.y;
			titleOrigin.y = a; // bounds.y + a;
			borderRect.y = X; // bounds.y + X;
			borderRect.height = bounds.height - X;
			break;
			
		case BELOWTOP:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset + b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = borderSize; // bounds.y + borderSize;
			titleOrigin.y = borderSize + a; // bounds.y + borderSize + a;
			borderRect.y = 0; // bounds.y;
			borderRect.height = bounds.height;
			break;
			
		case ABOVEBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + b; // + horizOffset
			titleRect.height = b;
			titleRect.y = bounds.height - insets.bottom;
// + bounds.y;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0; // bounds.y;
			borderRect.height = bounds.height;
			break;
			
		case ATBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			b = (int) Math.round (0.5834 * fontSize);
			X = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = X;
			titleRect.height = X;
			titleRect.y = bounds.height - X; // + bounds.y;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0; // bounds.y;
			borderRect.height = bounds.height - b;
			break;
			
		case BELOWBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			// b = (int) Math.round (0.5834 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset + b;
			titleRect.height = b;
			titleRect.y = bounds.height - b; // + bounds.y
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0; // bounds.y;
			borderRect.height = bounds.height - b; // F
			break;

		default: break; // ??
		}
	}
	
	public Dimension minimumSize ()
	{
		calculateGeometry ();
		return new Dimension (2 + insets.left + insets.right, 
							  2 + insets.top + insets.bottom);
	}
	
	public String title ()
	{
		return title;
	}
	
	public void setTitle (String newTitle)
	{
		title = newTitle;
	}
	
	public int titlePosition ()
	{
		return titlePosition;
	}
	
	public void setTitlePosition (int newPosition)
	{
		titlePosition = newPosition;
	}
	
	public int borderType ()
	{
		return borderType;
	}
	
	public void setBorderType (int newBorderType)
	{
		borderType = newBorderType;
	}
	
	public int horizOffset ()
	{
		return horizOffset;
	}
	
	public void setHorizOffset (int offset)
	{
		horizOffset = offset;
	}
	
	public int vertOffset ()
	{
		return vertOffset;
	}
	
	public void setVertOffset (int offset)
	{
		vertOffset = offset;
	}
	
	public Insets insets()
	{
		calculateGeometry ();
		return insets;
	}
	
	private void drawLineRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.black);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + r.height - 1, 
					r.x, r.y + r.height - 1);
		g.drawLine (r.x, r.y + r.height - 1, 
					r.x, r.y);
	}
	
	private void drawBezelRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 2, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 2);
		g.setColor (Color.black);
		g.drawLine (r.x + 2, r.y + 1,
					r.x + r.width - 3, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 3);
		g.setColor (Color.lightGray);
		g.drawLine (r.x + 1, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 1,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawGrooveRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + r.width - 1, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + 2,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawText (Graphics g, Point p, String text)
	{
		g.setFont (getFont ());
		g.setColor (getForeground ());
		g.drawString (text, p.x, p.y);
	}
	
	public void paint (Graphics g)
	{
		Rectangle bounds = bounds ();

		calculateGeometry ();

        // System.out.println ("calculated " + this); // DEBUG		
		// update does it 
		// g.clearRect (0 /*bounds.x*/, 0 /*bounds.y*/, bounds.width, bounds.height);

		switch (borderType) {
		case NOBORDER: break;
		case LINE:     drawLineRect (g, borderRect); break;
		case BEZEL:    drawBezelRect (g, borderRect); break;
		case GROOVE:   drawGrooveRect (g, borderRect); break;
		}
		
		if (titlePosition == NOTITLE)
		  return;
		  
		g.clearRect (titleRect.x, titleRect.y, 
					 titleRect.width, titleRect.height);

		if (title != null && title.length () > 0)
			drawText (g, titleOrigin, title);
	}

    public void paintComponents (Graphics g)
	{
		Rectangle bounds, cBounds;
		int ncomponents;
		
		ncomponents = countComponents ();
		bounds = bounds ();
		for (int i = 0 ; i < ncomponents ; i++) {
	    	Component comp = getComponent (i);
			if (comp != null) {
	    		cBounds = comp.bounds ();
				Graphics cg = g.create(cBounds.x, cBounds.y, cBounds.width, cBounds.height);
				g.clipRect (bounds.x - cBounds.x + insets.left, 
							bounds.y - cBounds.y + insets.top,
							bounds.width - insets.left - insets.right,
							bounds.height - insets.top - insets.bottom);
				try {
		    		comp.paintAll(cg);
				} finally {
		    		cg.dispose();
				}
	    	}
		}
    }
	
    public void paintAll(Graphics g)
	{
		if (isVisible ()) {
			validate ();
	    	paint (g);
			paintComponents (g);
		}
    }
	
	public String toString ()
	{
		return super.toString () +
				"title: " + title + "\n" +
				"titlePosition: " + titlePosition + "\n" +
				"borderType: " + borderType + "\n" +
				"horizOffset: " + horizOffset + "\n" +
				"vertOffset: " + vertOffset + "\n" +
				"insets: " + insets + "\n" +
				"titleRect: " + titleRect + "\n" +
				"borderRect: " + borderRect + "\n" +
				"titleOrigin: " + titleOrigin + "\n";
	}
}
