/*
 * Copyright (c) 1996 melonSoft Ralf Suckow Berlin. All Rights Reserved.
 *
 * MELONSOFT RALF SUCKOW BERLIN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. MELONSOFT RALF SUCKOW BERLIN 
 * SHALL NOT BE LIABLE FOR ANY DAMAGES RESULTING FROM USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package aCupOfCoffee;

import java.awt.*;

class TransparentPanel extends Panel {

	public void update (Graphics g)
	{
		paint (g);
	}
	public void paint (Graphics g)
	{
	}
}

class BoxBorder extends Canvas {

	public final static int NOBORDER = 0;
	public final static int LINE     = 1;
	public final static int BEZEL    = 2;
	public final static int GROOVE   = 3;
	
	public final static int NOTITLE     = 0;
	public final static int ABOVETOP    = 1;
	public final static int ATTOP       = 2;
	public final static int BELOWTOP    = 3;
	public final static int ABOVEBOTTOM = 4;
	public final static int ATBOTTOM    = 5;
	public final static int BELOWBOTTOM = 6;
	
	private final static int HORIZTITLEOFFSET = 4;
	
	// permanent attributes
	
	private String		title;
	private int			titlePosition;
	private int			borderType;
	private int			horizOffset;
	private int			vertOffset;
	
	// temporary attributes derived in layout
	private Insets		insets;
	private Rectangle	titleRect;
	private Rectangle	borderRect;
	private Point		titleOrigin;
	
	public BoxBorder (String theTitle, int theTitlePosition, 
					  int theBorderType,
int hOffset, int vOffset)
	{
		title = theTitle;
		titlePosition = theTitlePosition;
		borderType = theBorderType;
		horizOffset = hOffset;
		vertOffset = vOffset;
		insets = new Insets (0, 0, 0, 0);
		titleRect = new Rectangle ();
		borderRect = new Rectangle ();
		titleOrigin = new Point (0, 0);
		setBackground (Color.lightGray);
		setForeground (Color.black);
	}

	public BoxBorder ()
	{
		this ("Box", ATTOP, GROOVE, 5, 5);
	}

	public void layout ()
	{
		// we are calculating:
		// insets
		// titleRect
		// titleOrigin
		// borderRect
		
		// the input is the own bounds and the font size 
		// as well as the title
		
		Rectangle	bounds;
		int			borderSize;
		FontMetrics fontMetrics;
		Font		font;
		double      fontSize;
		int			ascent;
		int			a, b, X; // temporary values
		
		bounds = bounds ();
		font = getFont ();

		if (font == null)
			// not yet on the screen
			return;

		fontMetrics = getFontMetrics (font);
		if (fontMetrics == null)
		  return; // not yet on screen
		
		ascent = fontMetrics.getMaxAscent ();
		// 1.0 final: 
		// fontSize = (double) (fontMetrics.getMaxDescent () + ascent);
		// 1.0 prebeta 1:
		fontSize = (double) (fontMetrics.getMaxDecent () + ascent);

		switch (borderType) {
		case NOBORDER:	borderSize = 0; break;
		case LINE:		borderSize = 1; break;
		case BEZEL:		borderSize = 2; break;
		case GROOVE:	borderSize = 2; break;
		default:		borderSize = 0;
		}
		
		// horizontal settings
		insets.left = borderSize + horizOffset;
		insets.right = borderSize + horizOffset;
		titleRect.width = fontMetrics.stringWidth (title)
							+ 2 * HORIZTITLEOFFSET; // CHECK
		titleRect.x = (bounds.width - titleRect.width) / 2;
		titleOrigin.x = titleRect.x + HORIZTITLEOFFSET;
		borderRect.x = 0;
		borderRect.width = bounds.width;
		
		// vertical settings
		switch (titlePosition) {
		case NOTITLE:
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = 0;
			titleRect.y = 0;
			titleOrigin.y = 0;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ABOVETOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			
			insets.top = b + borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0;
			titleOrigin.y = a;
			borderRect.y = b;
			borderRect.height = bounds.height - b;
			break;
			
		case ATTOP:
			a = (int) Math.round (0.95 * fontSize);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
			X = (int) Math.round (0.5834 * fontSize);
			
			insets.top = b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = 0;
			titleOrigin.y = a;
			borderRect.y = X;
			borderRect.height = bounds.height - X;
			break;
			
		case BELOWTOP:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset + b;
			insets.bottom = borderSize + vertOffset;
			titleRect.height = b;
			titleRect.y = borderSize;
			titleOrigin.y = borderSize + a;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ABOVEBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			if (fontSize < 8.0)
				b = (int) Math.round (1.33333333 * fontSize);
			else
				b = (int) Math.round (1.4 * fontSize + 0.5);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + b;
			titleRect.height = b;
			titleRect.y = bounds.height - insets.bottom;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height;
			break;
			
		case ATBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			b = (int) Math.round (0.5834 * fontSize);
			X = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = X;
			titleRect.height = X;
			titleRect.y = bounds.height - X;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height - b;
			break;
			
		case BELOWBOTTOM:
			a = (int) Math.round (1.05555555 * fontSize + 0.1);
			b = (int) Math.round (1.28125 * fontSize + 0.7);
				
			insets.top = borderSize + vertOffset;
			insets.bottom = borderSize + vertOffset + b;
			titleRect.height = b;
			titleRect.y = bounds.height - b;
			titleOrigin.y = titleRect.y + a;
			borderRect.y = 0;
			borderRect.height = bounds.height - b;
			break;

		default: break; // ??
		}
	}
	
	public Dimension minimumSize ()
	{
		
		return new Dimension (2 + insets.left + insets.right, 
							  2 + insets.top + insets.bottom);
	}
	
	public String title ()
	{
		return title;
	}
	
	public void setTitle (String newTitle)
	{
		invalidate ();
		title = newTitle;
	}
	
	public int titlePosition ()
	{
		return titlePosition;
	}
	
	public void setTitlePosition (int newPosition)
	{
		invalidate ();
		titlePosition = newPosition;
	}
	
	public int borderType ()
	{
		return borderType;
	}
	
	public void setBorderType (int newBorderType)
	{
		invalidate ();
		borderType = newBorderType;
	}
	
	public int horizOffset ()
	{
		return horizOffset;
	}
	
	public void setHorizOffset (int offset)
	{
		invalidate ();
		horizOffset = offset;
	}
	
	public int vertOffset ()
	{
		return vertOffset;
	}
	
	public void setVertOffset (int offset)
	{
		invalidate ();
		vertOffset = offset;
	}
	
	public Insets insets()
	{
		validate ();
		return insets;
	}
	
	private void drawLineRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.black);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + r.height - 1, 
					r.x, r.y + r.height - 1);
		g.drawLine (r.x, r.y + r.height - 1, 
					r.x, r.y);
	}
	
	private void drawBezelRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 2, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 2);
		g.setColor (Color.black);
		g.drawLine (r.x + 2, r.y + 1,
					r.x + r.width - 3, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 3);
		g.setColor (Color.lightGray);
		g.drawLine (r.x + 1, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 1,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawGrooveRect (Graphics g, Rectangle r)
	{
		g.setColor (Color.darkGray);
		g.drawLine (r.x, r.y,
					r.x + r.width - 1, r.y);
		g.drawLine (r.x, r.y,
					r.x, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine (r.x + r.width - 2, r.y + 2,
					r.x + r.width - 2, r.y + r.height - 2);
		g.setColor (Color.white);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + r.width - 1, r.y + 1);
		g.drawLine (r.x + 1, r.y + 1,
					r.x + 1, r.y + r.height - 1);
		g.drawLine (r.x + 2, r.y + r.height - 1,
					r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine (r.x + r.width - 1, r.y + 2,
					r.x + r.width - 1, r.y + r.height - 1);
	}
	
	private void drawText (Graphics g, Point p, String text)
	{
		g.setFont (getFont ());
		g.setColor (Color.black);
		g.drawString (text, p.x, p.y);
	}
	
	public void paint (Graphics g)
	{



		switch (borderType) {
		case NOBORDER: break;
		case LINE:     drawLineRect (g, borderRect); break;
		case BEZEL:    drawBezelRect (g, borderRect); break;
		case GROOVE:   drawGrooveRect (g, borderRect); break;
		}
		
		if (titlePosition == NOTITLE)
		  return;
		  
		g.clearRect (titleRect.x, titleRect.y, 
					 titleRect.width, titleRect.height);

		if (title != null && title.length () > 0)
			drawText (g, titleOrigin, title);
	}
	
	public void update (Graphics g)
	{
	  paint (g);
	}
}

public class Box extends aCupOfCoffee.TransparentPanel {

	public final static String copyright =
			"(c) 1996 melonSoft Ralf Suckow Berlin. All Rights Reserved";  
	
	private aCupOfCoffee.BoxBorder border;
	private aCupOfCoffee.TransparentPanel content;
	
	public Box (String title, int titlePosition, 
				int borderType,
int hOffset, int vOffset)
	{
		super.setLayout (null);
		content = new aCupOfCoffee.TransparentPanel ();
		super.add (content);
		border = new aCupOfCoffee.BoxBorder (title, titlePosition,
											 borderType, hOffset, vOffset);
		super.add (border);
	}

	public Component add (Component comp)
	{
		if (content == null)
			return super.add (comp);
		else
			return content.add (comp);
	}
	
	public void setLayout(LayoutManager mgr)
	{
		if (content == null)
			super.setLayout (mgr);
		else
			content.setLayout (mgr);
	}

	public Box ()
	{
		this ("Box", BoxBorder.ATTOP, BoxBorder.GROOVE, 5, 5);
	}

	public void layout ()
	{
		
		Rectangle bounds;
		Insets insets;
		
		border.layout ();
		bounds = bounds ();
		insets = border.insets ();
		content.reshape (insets.left, insets.top,
						 bounds.width - insets.left - insets.right,
						 bounds.height - insets.top - insets.bottom);
		content.layout ();
	}	
	
	public Dimension minimumSize ()
	{
	
		Dimension contentSize;
		Insets insets;
		
		contentSize = content.minimumSize ();	
		insets = border.insets ();
		return new Dimension (contentSize.width + insets.left + insets.right,
						 contentSize.height + insets.top + insets.bottom);
	}
	
	public Dimension preferredSize ()
	{
	
		Dimension contentSize;
		Insets insets;
		
		contentSize = content.preferredSize ();	
		insets = border.insets ();
		return new Dimension (contentSize.width + insets.left + insets.right,
						 contentSize.height + insets.top + insets.bottom);
	}
}
